@@ -6,17 +6,18 @@
  * SPDX-License-Identifier: MIT
  */
 
-#define G_LOG_DOMAIN "wp-object-interest"
-
 #include "object-interest.h"
 #include "global-proxy.h"
 #include "session-item.h"
 #include "proxy-interfaces.h"
+#include "event-dispatcher.h"
 #include "log.h"
 #include "error.h"
 
 #include <pipewire/pipewire.h>
 
+WP_DEFINE_LOCAL_LOG_TOPIC ("wp-object-interest")
+
 /*! \defgroup wpobjectinterest WpObjectInterest */
 /*!
  * \struct WpObjectInterest
@@ -80,7 +81,7 @@ G_DEFINE_BOXED_TYPE (WpObjectInterest, wp_object_interest,
  * For further reading on the constraint's arguments, see
  * wp_object_interest_add_constraint()
  *
- * For example, this interest matches objects that are descendands of WpProxy
+ * For example, this interest matches objects that are descendants of WpProxy
  * with a "bound-id" between 0 and 100 (inclusive), with a pipewire property
  * called "format.dsp" that contains the string "audio" somewhere in the value
  * and with a pipewire property "port.name" being present (with any value):
@@ -312,7 +313,8 @@ wp_object_interest_validate (WpObjectInterest * self, GError ** error)
     return TRUE;
 
   if (!G_TYPE_IS_OBJECT (self->gtype) && !G_TYPE_IS_INTERFACE (self->gtype) &&
-          !g_type_is_a (self->gtype, WP_TYPE_PROPERTIES)) {
+          !g_type_is_a (self->gtype, WP_TYPE_PROPERTIES) &&
+          !g_type_is_a (self->gtype, WP_TYPE_EVENT)) {
     g_set_error (error, WP_DOMAIN_LIBRARY, WP_LIBRARY_ERROR_INVARIANT,
         "type '%s' is not a valid interest type", g_type_name (self->gtype));
     return FALSE;
@@ -761,13 +763,15 @@ wp_object_interest_matches_full (WpObjectInterest * self,
       WpObject *oo = (WpObject *) object;
       WpPipewireObject *pwo = (WpPipewireObject *) object;
 
-      if (wp_object_get_active_features (oo) & WP_PIPEWIRE_OBJECT_FEATURE_INFO)
+      if (wp_object_test_active_features (oo, WP_PIPEWIRE_OBJECT_FEATURE_INFO))
         pw_props = props = wp_pipewire_object_get_properties (pwo);
     }
 
     if (!pw_global_props && WP_IS_SESSION_ITEM (object)) {
       WpSessionItem *si = (WpSessionItem *) object;
       pw_global_props = props = wp_session_item_get_properties (si);
+      if (!pw_props)
+        pw_props = props;
     }
   }
 
@@ -877,3 +881,51 @@ wp_object_interest_matches_full (WpObjectInterest * self,
   }
   return result;
 }
+
+/*!
+ * \brief Finds all the defined constraint values for a subject in \a self.
+ *
+ * A defined constraint value is the value of a constraint with the 'equal' or
+ * 'in-list' verb, because the full value must be defined with those verbs. This
+ * can be useful for cases where we want to enumerate interests that are
+ * interested in specific subjects.
+ *
+ * \ingroup wpobjectinterest
+ * \param self the object interest
+ * \param type the constraint type
+ * \param subject the subject that the constraint applies to
+ * \returns (element-type GVariant) (transfer full) (nullable): the defined
+ * constraint values for this object interest.
+ * \since 0.5.13
+ */
+GPtrArray *
+wp_object_interest_find_defined_constraint_values (WpObjectInterest * self,
+    WpConstraintType type, const gchar * subject)
+{
+  GPtrArray *res = g_ptr_array_new_with_free_func (
+      (GDestroyNotify)g_variant_unref);
+  struct constraint *c;
+
+  pw_array_for_each (c, &self->constraints) {
+    if ((c->type == type || WP_CONSTRAINT_TYPE_NONE == type) &&
+        g_str_equal (c->subject, subject)) {
+      switch (c->verb) {
+        case WP_CONSTRAINT_VERB_EQUALS:
+          g_ptr_array_add (res, g_variant_ref (c->value));
+          break;
+        case WP_CONSTRAINT_VERB_IN_LIST: {
+          GVariantIter iter;
+          GVariant *child;
+          g_variant_iter_init (&iter, c->value);
+          while ((child = g_variant_iter_next_value (&iter)))
+            g_ptr_array_add (res, child);
+          break;
+        }
+        default:
+          break;
+      }
+    }
+  }
+
+  return res;
+}
